/* Rwv2 -- Microsoft Word Parser extension
   Copyright (C) 2003 Hannes Wyss, ywesee

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   ywesee - intellectual capital connected
	 Winterthurerstrasse 52, CH-8006 Zürich, Switzerland
   hwyss@ywesee.com
*/

#ifndef RWV2_ASSOCIATED_STRINGS_H
#define RWV2_ASSOCIATED_STRINGS_H

#include "ustring.h"
#include "associatedstrings.h"

namespace Rwv2 {
class AssociatedStrings {
	public:
	const char * title;
	AssociatedStrings(wvWare::AssociatedStrings * strings) {
		wvWare::UString utitle = strings->title();
		wvWare::CString ctitle = utitle.cstring();
		title = ctitle.c_str();
	};
};
} // namespace Rwv2
#endif
